@@ -7,7 +7,7 @@ from datetime import datetime, timedelta
 from typing import Dict, List, Optional, Tuple, Any
 from . import typings
 from .software_version import SoftwareVersion
-from .exceptions import CredentialsInvalidError, SnapshotIsNotValidFileTypeError, InvalidContentTypeError
+from .exceptions import CredentialsInvalidError, SnapshotIsNotValidFileTypeError, InvalidContentTypeError, ApiError
 import traceback
 import re
 
@@ -48,6 +48,7 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
         timeout=DEFAULT_TIMEOUT,
         stream_format=DEFAULT_STREAM_FORMAT,
         rtmp_auth_method=DEFAULT_RTMP_AUTH_METHOD,
+        aiohttp_get_session_callback=None
     ):
         """Initialize the API class."""
         self._url = ""
@@ -119,8 +120,14 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
         self._is_nvr = False
         self._is_ia_enabled = False
 
-        self._aiohttp_session: aiohttp.ClientSession = aiohttp.ClientSession(timeout=self._timeout,
+        if aiohttp_get_session_callback is not None:
+            self._aiohttp_get_session_callback = aiohttp_get_session_callback
+            self._aiohttp_session = None
+        else:
+            self._aiohttp_session: aiohttp.ClientSession = aiohttp.ClientSession(timeout=self._timeout,
                                                                     connector=aiohttp.TCPConnector(ssl=False))
+            self._aiohttp_get_session_callback = None
+
 
         self._api_version_getrec: int = 0
         self._api_version_getftp: int = 0
@@ -599,16 +606,25 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
                 {"cmd": "GetAiState", "action": 0, "param": {"channel": self._channel}}]
 
         response = await self.send(body)
-        json_data = json.loads(response)
+        if response is None:
+            return False
 
-        if json_data is None:
-            _LOGGER.error(
-                "Unable to get All Motion States at IP %s", self._host
-            )
+        try:
+            json_data = json.loads(response)
+
+            if json_data is None:
+                _LOGGER.error(
+                    "Unable to get All Motion States at IP %s", self._host
+                )
+                self._motion_state = False
+                return self._motion_state
+
+            self.map_json_response(json_data)
+        except (TypeError, json.JSONDecodeError):
+            self.clear_token()
             self._motion_state = False
-            return self._motion_state
 
-        self.map_json_response(json_data)
+        return self._motion_state
 
     async def get_still_image(self):
         """Get the still image."""
@@ -890,7 +906,8 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
 
         await self.send(body, param)
         self.clear_token()
-        await self._aiohttp_session.close()
+        if self._aiohttp_session is not None:
+            await self._aiohttp_session.close()
 
     async def set_channel(self, channel):
         """Update the channel property."""
@@ -1636,7 +1653,7 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
     async def send(self, body, param=None, expected_content_type: Optional[str] = None):
         """Generic send method."""
 
-        if self._aiohttp_session.closed:
+        if self._aiohttp_session is not None and self._aiohttp_session.closed:
             self._aiohttp_session = aiohttp.ClientSession(timeout=self._timeout,
                                                             connector=aiohttp.TCPConnector(ssl=False))
 
@@ -1650,20 +1667,28 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
             param["token"] = self._token
 
         try:
+            session = self._aiohttp_session
+            if session is None:
+                session = self._aiohttp_get_session_callback()
             if body is None:
-                async with self._aiohttp_session.get(url=self._url, params=param, allow_redirects=False) as response:
+                async with session.get(url=self._url, params=param, allow_redirects=False) as response:
                     _LOGGER.debug("%s/%s::send() HTTP Request params =%s", self.name, self._host,
                                   str(param).replace(self._password, "<password>"))
-                    json_data = await response.read()
+
                     _LOGGER.debug("%s/%s::send() HTTP Response status=%s content-type=(%s)", self.name, self._host,
                                   response.status, response.content_type)
 
+                    json_data = await response.read()
                     if param.get("cmd") == "Snap":
                         _LOGGER_DATA.debug("%s/%s::send() HTTP Response data scrapped because it's too large",
                                            self.name, self._host)
                     else:
                         _LOGGER_DATA.debug("%s/%s::send() HTTP Response data: %s", self.name, self._host, json_data)
 
+                    if response.status >= 400:
+                        raise ApiError(
+                            "API returned HTTP status ERROR code {}/{}".format(response.status, response.reason))
+
                     if len(json_data) < 500 and response.content_type == 'text/html':
                         if b'"detail" : "invalid user"' in json_data or \
                                 b'"detail" : "login failed"' in json_data \
@@ -1677,22 +1702,27 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
 
                     return json_data
             else:
-                async with self._aiohttp_session.post(
+                async with session.post(
                         url=self._url, json=body, params=param, allow_redirects=False
                 ) as response:
                     _LOGGER.debug("%s/%s::send() HTTP Request params =%s", self.name, self._host,
                                   str(param).replace(self._password, "<password>"))
                     _LOGGER.debug("%s/%s::send() HTTP Request body =%s", self.name, self._host,
                                   str(body).replace(self._password, "<password>"))
-                    json_data = await response.text()
                     _LOGGER.debug("%s/%s::send() HTTP Response status=%s content-type=(%s)", self.name, self._host,
                                   response.status, response.content_type)
+
+                    json_data = await response.text()
                     if param.get("cmd") == "Search" and len(json_data) > 500:
                         _LOGGER_DATA.debug("%s/%s::send() HTTP Response data scrapped because it's too large",
                                            self.name, self._host)
                     else:
                         _LOGGER_DATA.debug("%s/%s::send() HTTP Response data: %s", self.name, self._host, json_data)
 
+                    if response.status >= 400:
+                        raise ApiError(
+                            "API returned HTTP status ERROR code {}/{}".format(response.status, response.reason))
+
                     if len(json_data) < 500 and response.content_type == 'text/html':
                         if 'detail" : "invalid user' in json_data or 'detail" : "login failed' in json_data \
                                 or 'detail" : "please login first' in json_data:
